==> asynctools-0.0.1-0.linux-x86_64.tar.gz: media/0DAY/Projects/2014/lib/Python/asynctools/env/lib/python2.7/site-packages/asynctools/threading/Parallel.py <==
from threading import Thread, Event
from Queue import Queue


class Parallel(object):
    """ Execute all tasks in parallel, once """
    _class = 'Parallel'

    def __init__(self, worker):
        """ Initialize threaded executor with a worker function
            :param worker: A function to process a single job. Arguments are passed from :meth:Once.job
            :type worker: callable
        """
        self._worker = worker
        self._jobs = Queue()
        self._results = []
        self._errors = []

    def _spawn_thread(self, target):
        """ Create a thread """
        t = Thread(target=target)
        t.daemon = True
        t.start()
        return t

    def _terminate_thread(self):
        """ Put (None,None) in the queue, which is the thread termination signal """
        self._jobs.put((None, None))  # Termination signal

    def _worker_once(self):
        """ Worker wrapper that stores results """
        # Get
        args, kwargs = self._jobs.get()

        # Stop thread when (None, None) comes in
        if args is None and kwargs is None:
            return False  # Top-level loopers should exit as well

        # Work
        try:
            self._results.append(self._worker(*args, **kwargs))
        except Exception as e:
            self._errors.append(e)
        finally:
            self._jobs.task_done()

    def job(self, *args, **kwargs):
        """ Add a job. Arguments are directly passed to the worker """
        self._jobs.put((args, kwargs))

        # Spawn a thread if needed
        if self._class == 'Parallel':
            self._spawn_thread(self._worker_once)

    def join(self):
        """ Wait for all current tasks to be finished """
        self._jobs.join()
        try:
            return self._results, self._errors
        finally:
            self._clear()

    def _clear(self):
        self._results = []
        self._errors = []

==> asynctools-0.1.0-0.linux-x86_64.tar.gz: media/0DAY/Projects/2014/lib/Python/asynctools/env/lib/python2.7/site-packages/asynctools/threading/Parallel.py <==
from threading import Thread, Event
from Queue import Queue


class Parallel(object):
    """ Execute all tasks in parallel, once """
    _class = 'Parallel'

    def __init__(self, worker):
        """ Initialize threaded executor with a worker function
            :param worker: A function to process a single job. Arguments are passed from :meth:Once.job
            :type worker: callable
        """
        self._worker = worker
        self._jobs = Queue()
        self._results , self._errors = [], []

    def _spawn_thread(self, target):
        """ Create a thread """
        t = Thread(target=target)
        t.daemon = True
        t.start()
        return t

    def _add_job(self, args, kwargs):
        self._jobs.put((args, kwargs))

    def _terminate_one_thread(self):
        """ Put (None,None) in the queue, which is the thread termination signal """
        self._add_job(None, None)

    def _thread(self):
        """ Thread entry point: does the job once, stored results, and dies. """
        # Get
        args, kwargs = self._jobs.get()

        # Stop thread when (None, None) comes in
        if args is None and kwargs is None:
            return False  # Top-level loopers should exit as well

        # Work
        try:
            self._results.append(self._worker(*args, **kwargs))
        except Exception as e:
            self._errors.append(e)
        finally:
            self._jobs.task_done()

    def map(self, jobs):
        map(self, jobs)
        return self

    def __call__(self, *args, **kwargs):
        """ Add a job. Arguments are directly passed to the worker """
        self._add_job(args, kwargs)
        if self._class == 'Parallel':
            self._spawn_thread(self._thread)
        return self

    def join(self):
        """ Wait for all current tasks to be finished """
        self._jobs.join()
        try:
            return self._results, self._errors
        finally:
            self._clear()

    def _clear(self):
        self._results = []
        self._errors = []
